@@ -16,6 +16,47 @@ List rxModelVars_(const RObject &obj);
 bool rxIs(const RObject &obj, std::string cls);
 Environment RxODEenv();
 
+Environment dataTable;
+bool getForder_b=false;
+Function getRxFn(std::string name);
+bool dtForder = false;
+bool forderForceBase_ = false;
+//' Force using base order for RxODE radix sorting
+//'
+//' @param forceBase boolean indicating if RxODE should use R's
+//'   \code{\link{order}} for radix sorting instead of
+//'   \code{data.table}'s parallel radix sorting.
+//'
+//' @examples
+//' \dontrun{
+//' forderForceBase(TRUE) # Use base `order` for RxODE sorts
+//' forderForceBase(FALSE) # Use base `data.table` for RxODE sorts
+//' }
+//'@export
+//[[Rcpp::export]]
+RObject forderForceBase(bool forceBase = false){
+  forderForceBase_=forceBase;
+  return R_NilValue;
+}
+Function getForder(){
+  if (!getForder_b){
+    Function fn = getRxFn(".getDTEnv");
+    dataTable = fn();
+    getForder_b=true;
+  }
+  if (!forderForceBase_ && dataTable.exists("forder")){
+    dtForder=true;
+    return dataTable["forder"];
+  }
+  Environment b=Rcpp::Environment::base_namespace();
+  dtForder=false;
+  return b["order"];
+}
+
+extern bool useForder(){
+  return getForder_b;
+}
+
 IntegerVector toCmt(RObject inCmt, CharacterVector& state, const bool isDvid,
 		    const int stateSize, const int sensSize, IntegerVector& curDvid){
   RObject cmtInfo = R_NilValue;
@@ -239,6 +280,27 @@ bool rxSetIni0(bool ini0 = true){
 extern void setFkeep(List keep);
 IntegerVector convertMethod(RObject method);
 
+bool useRadix_=true;
+
+extern bool useRadix(){
+  return useRadix_;
+}
+
+//' Use Radix Sort when possible
+//'
+//' By default RxODE uses radix sort when possible.
+//'
+//' @param useRadix Use \code{order} with \code{method} = \code{radix}
+//'   when appropriate.  Otherwise use \code{timsort}.
+//'
+//' @export
+//' @author Matthew Fidler
+//[[Rcpp::export]] 
+bool rxUseRadixSort(bool useRadix = true){
+  useRadix_=useRadix;
+  return useRadix_;
+}
+
 bool warnedNeg=false;
 //' Event translation for RxODE
 //'
@@ -263,12 +325,13 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
 	     bool dropUnits=false, bool allTimeVar=false,
 	     bool keepDosingOnly=false, Nullable<LogicalVector> combineDvid=R_NilValue,
 	     CharacterVector keep = CharacterVector(0)){
+  // clock_t _lastT0 = clock();
   Environment rx = RxODEenv();
   bool combineDvidB = false;
+  Environment b=Rcpp::Environment::base_namespace();
   if (!combineDvid.isNull()){
     combineDvidB = (as<LogicalVector>(combineDvid))[1];
   } else {
-    Environment b=Rcpp::Environment::base_namespace();
     Function getOption = b["getOption"];
     combineDvidB = as<bool>(getOption("RxODE.combine.dvid", true));
   }
@@ -314,6 +377,8 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
     }
     // stop("This dataset was prepared for another model.");
   }
+  // REprintf("Time1: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
   // Translates events + model into translated events
   CharacterVector dName = as<CharacterVector>(inData.attr("names"));
   CharacterVector lName = clone(dName);
@@ -390,6 +455,8 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       }
     }
   }
+  // REprintf("Time2: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
   if ((int)(keepCol.size())!=(int)keep.size()){
     std::string wKeep = "Cannot keep missing columns:";
     for (j = 0; j < keep.size(); j++){
@@ -403,10 +470,11 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
   CharacterVector covUnitsN(covCol.size());
   NumericVector nvTmp, nvTmp2;
   bool hasCmt = false;
+  int cmtI =0;
   for (i = covCol.size(); i--;){
     covUnitsN[i] = lName[covCol[i]];
     nvTmp2 = NumericVector::create(1.0);
-    if (as<std::string>(lName[covCol[i]]) != "cmt"){
+    if (hasCmt || as<std::string>(lName[covCol[i]]) != "cmt"){
       nvTmp = as<NumericVector>(inData[covCol[i]]);
       if (!dropUnits && rxIs(nvTmp, "units")){
 	nvTmp2.attr("class") = "units";
@@ -414,6 +482,7 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       }
     } else {
       hasCmt=true;
+      cmtI = i;
     }
     covUnits[i] = nvTmp2;
   }
@@ -479,20 +548,6 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       curDvid[i] = stateS.size()+curDvid[i];
     }
   }
-  std::vector<int> id;
-  std::vector<int> allId;
-  std::vector<int> obsId;
-  std::vector<int> zeroId;
-  std::vector<int> doseId;
-  std::vector<int> evid;
-  std::vector<double> time;
-  std::vector<double> amt;
-  std::vector<double> ii;
-  std::vector<int> idx;
-  std::vector<int> cmtF;
-  std::vector<int> dvidF;
-  std::vector<double> dv;
-  std::vector<int> idxO;
   if (timeCol== -1){
     stop("time is required in dataset.");
   }
@@ -505,6 +560,39 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
     newInData =  convDate(newInData);
     return etTrans(newInData, obj, addCmt, dropUnits, allTimeVar, keepDosingOnly, combineDvid);
   }
+  size_t resSize = inTime.size()+256;
+  std::vector<int> id;
+  id.reserve(resSize);
+  std::vector<int> allId;
+  allId.reserve(resSize);
+  std::vector<int> obsId;
+  obsId.reserve(resSize);
+  std::vector<int> zeroId;
+  obsId.reserve(resSize);
+  std::vector<int> doseId;
+  doseId.reserve(resSize);
+  std::vector<int> evid;
+  evid.reserve(resSize);
+  std::vector<double> time;
+  time.reserve(resSize);
+  std::vector<double> amt;
+  amt.reserve(resSize);
+  std::vector<double> ii;
+  ii.reserve(resSize);
+  std::vector<int> idx;
+  idx.reserve(resSize);
+  std::vector<int> cmtF; // Final compartment
+  cmtF.reserve(resSize);
+  std::vector<int> dvidF;
+  dvidF.reserve(resSize);
+  std::vector<double> dv;
+  dv.reserve(resSize);
+  std::vector<int> idxO;
+  idxO.reserve(resSize);
+
+  // REprintf("Time3: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   // save units information
   bool addTimeUnits = false;
   RObject timeUnits;
@@ -550,10 +638,12 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
   }
   IntegerVector inEvid;
   bool evidIsMDV = false;
+  bool hasEvid=false;
   if (evidCol != -1){
     if (rxIs(inData[evidCol], "integer") || rxIs(inData[evidCol], "numeric") ||
 	rxIs(inData[evidCol], "logical")){
       inEvid = as<IntegerVector>(inData[evidCol]);
+      hasEvid=true;
     } else {
       stop("Event id (evid) needs to be an integer");
     }
@@ -567,8 +657,11 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
     } else {
       stop("Missing DV (mdv) needs to be an integer");
     }
+    hasEvid=true;
   } else if (methodCol != -1){
     inEvid = convertMethod(inData[methodCol]);
+    // hasEvid=true; The EVID is not present in the right form.
+    // This allows mixing of deSolve with rate info
   }
   IntegerVector inMdv;
   if (mdvCol != -1){
@@ -661,6 +754,10 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
   double cdv;
   int nobs=0, ndose=0;
   bool doWarnNeg=false;
+  
+  // REprintf("Time4: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   for (int i = 0; i < inTime.size(); i++){
     if (idCol == -1) cid = 1;
     else cid = inId[i];
@@ -1051,11 +1148,11 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       if (cevid > 10 && cevid < 99){
 	continue;
       }
-      if (rateI != 0){
+      if (rateI != 0 && hasEvid){
 	warning("'rate' or 'dur' is ignored with classic RxODE EVIDs");
 	rateI = 0;
       }
-      if (flg!=1){ // ss=1 is the same as ss=0 for NONMEM
+      if (flg!=1 && hasEvid){ // ss=1 is the same as ss=0 for NONMEM
 	warning("'ss' is ignored with classic RxODE EVIDs.");
 	flg=1;
       }
@@ -1162,6 +1259,10 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       }
     }
   }
+  
+  // REprintf("Time5: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   bool redoId=false;
   if (!keepDosingOnly){
     if (obsId.size() != allId.size()){
@@ -1176,6 +1277,10 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       redoId=true;
     }
   }
+
+  // REprintf("Time6: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   if (zeroId.size() != allId.size()){
     std::string idWarn = "IDs without zero-time start at the first observed time:";
     for (j = allId.size(); j--;){
@@ -1206,46 +1311,98 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
     }
     if (!_ini0) warning(idWarn.c_str());
   }
-  SORT(idxO.begin(),idxO.end(),
-       [id,time,evid,amt,doseId,keepDosingOnly](int a, int b){
-	 if (id[a] == id[b]){
-	   if (time[a] == time[b]){
-	     if (evid[a] == evid[b]){
-	       return a < b;
+
+  // REprintf("Time7: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  if (useRadix_){
+    IntegerVector ivId = wrap(id);
+    NumericVector nvTime = wrap(time);
+    IntegerVector ivEvid = clone(wrap(evid));
+    if (!keepDosingOnly && doseId.size() > 0){
+#define sortID if (ivEvid[j]==3){ \
+	  ivEvid[j] = NA_INTEGER+1; \
+	} else if (amt[j] == 0){ \
+	   ivEvid[j] = NA_INTEGER+2; \
+	} else { \
+	  ivEvid[j] = -ivEvid[j]; \
+	}
+      for (int j = ivId.size(); j--; ){
+	if (!(std::find(doseId.begin(), doseId.end(), ivId[j]) == doseId.end())){
+	  ivId[j] = NA_INTEGER; // Drop
+	}
+	// Duplicated below for speed.
+	sortID
+      }
+    } else {
+      for (int j = ivEvid.size(); j--; ){
+	sortID
+      }
+    }
+#undef sortID
+    Function order = getForder();
+    IntegerVector ord;
+    if (useForder()){
+      ord = order(ivId, nvTime, ivEvid,
+		  _["na.last"] = LogicalVector::create(true));
+      ord = ord - 1;
+      // na.last isn't =NA isn't quite working
+      idxO = as<std::vector<int>>(ord);
+      while (idxO.size() > 0 && IntegerVector::is_na(ivId[idxO.back()])){
+	idxO.pop_back();
+      }
+    } else {
+      ord = order(ivId, nvTime, ivEvid,
+		  _["na.last"] = LogicalVector::create(NA_LOGICAL),
+		  _["method"]="radix");
+      ord = ord - 1;
+      idxO = as<std::vector<int>>(ord);
+    }
+  } else {
+    SORT(idxO.begin(),idxO.end(),
+	 [id,time,evid,amt,doseId,keepDosingOnly](int a, int b){
+	   if (id[a] == id[b]){
+	     if (time[a] == time[b]){
+	       if (evid[a] == evid[b]){
+		 return a < b;
+	       }
+	       if (evid[a] == 3){
+		 return true;
+	       }
+	       if (evid[b] == 3){
+		 return false;
+	       }
+	       // Zero amts turn on and off compartments and should be first.
+	       if (evid[a] != 0 && amt[a] == 0){
+		 return true;
+	       }
+	       if (evid[b] != 0 && amt[b] == 0){
+		 return false;
+	       }
+	       return evid[a] > evid[b];
 	     }
-	     if (evid[a] == 3){
-	       return true;
-	     }
-	     if (evid[b] == 3){
+	     return time[a] < time[b];
+	   }
+	   // Bad IDs are pushed to the end to be popped off.
+	   if (!keepDosingOnly){
+	     if (doseId.size() > 0 && !(std::find(doseId.begin(), doseId.end(), id[a]) == doseId.end())){
 	       return false;
 	     }
-	     // Zero amts turn on and off compartments and should be first.
-	     if (evid[a] != 0 && amt[a] == 0){
+	     if (doseId.size() > 0 && !(std::find(doseId.begin(), doseId.end(), id[b]) == doseId.end())){
 	       return true;
 	     }
-	     if (evid[b] != 0 && amt[b] == 0){
-	       return false;
-	     }
-	     return evid[a] > evid[b];
 	   }
-	   return time[a] < time[b];
-	 }
-	 // Bad IDs are pushed to the end to be popped off.
-	 if (!keepDosingOnly){
-	   if (doseId.size() > 0 && !(std::find(doseId.begin(), doseId.end(), id[a]) == doseId.end())){
-	     return false;
-	   }
-	   if (doseId.size() > 0 && !(std::find(doseId.begin(), doseId.end(), id[b]) == doseId.end())){
-	     return true;
-	   }
-	 }
-	 return id[a] < id[b];
-       });
-  if (!keepDosingOnly && doseId.size() > 0){
-    while (idxO.size() > 0 && std::find(doseId.begin(), doseId.end(), id[idxO.back()]) != doseId.end()){
-      idxO.pop_back();
+	   return id[a] < id[b];
+	 });
+    if (!keepDosingOnly && doseId.size() > 0){
+      while (idxO.size() > 0 && std::find(doseId.begin(), doseId.end(), id[idxO.back()]) != doseId.end()){
+	idxO.pop_back();
+      }
     }
   }
+  // REprintf("Time8: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
+  
   if (idxO.size()==0) stop("Empty data.");
   int lastId = id[idxO.back()]+42;
   int rmAmt = 0;
@@ -1281,6 +1438,10 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
   } else {
     baseSize = 6;
   }
+  
+  // REprintf("Time9: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   List lst = List(baseSize+covCol.size());
   std::vector<bool> sub0(baseSize+covCol.size(), true);
   CharacterVector nme(baseSize+covCol.size());
@@ -1317,7 +1478,7 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
   nme1[0] = "ID";
   
   for (j = 0; j < (int)(covCol.size()); j++){
-    if (as<std::string>(lName[covCol[j]]) == "cmt"){
+    if (hasCmt && j == cmtI){
       lst[baseSize+j] = IntegerVector(idxO.size()-rmAmt);
       nme[baseSize+j] = pars[covParPos[j]];
       sub0[baseSize+j] = false;
@@ -1340,6 +1501,9 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
     keepN[j] = dName[keepCol[j]];
   }
 
+  // REprintf("Time10: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+
   IntegerVector ivTmp;
   lastId = NA_INTEGER;
   bool addId = false, added=false;
@@ -1398,7 +1562,7 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
 	}
       }
       for (j = 0; j < (int)(covCol.size()); j++){
-	if (as<std::string>(lName[covCol[j]]) == "cmt"){
+	if (hasCmt && j == cmtI){
 	  ivTmp = as<IntegerVector>(lst[baseSize+j]);
 	  ivTmp[jj] = cmtF[idxO[i]];
 	  if (!cmtFadd){
@@ -1443,6 +1607,10 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       }
     }
   }
+
+  // REprintf("Time11: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   if (!dropUnits && addTimeUnits){
     NumericVector tmpN = as<NumericVector>(lst[1]);
     tmpN.attr("class") = "units";
@@ -1485,6 +1653,8 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
   if (covCol.size() == 0 && !rxIs(lst1F[0], "integer") && !redoId){
     stop("Corrupted event table");
   }
+  // REprintf("Time12: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
   IntegerVector tmp = lst1F[0];
   CharacterVector idLvl2;
   if (redoId){
@@ -1497,6 +1667,9 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
     tmp.attr("levels") = R_NilValue;
     lst1F[0] = tmp;
   }
+
+  // REprintf("Time13: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
   
   lst1F.attr("names") = nme1F;
   lst1F.attr("class") = CharacterVector::create("data.frame");
@@ -1563,6 +1736,10 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       tmp.attr("levels") = idLvl;
     }
   }
+
+  // REprintf("Time14: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
+  
   lstF.attr("names") = nmeF;
   lstF.attr("class") = cls;
   lstF.attr("row.names") = IntegerVector::create(NA_INTEGER,-idxO.size()+rmAmt);
@@ -1572,5 +1749,7 @@ List etTrans(List inData, const RObject &obj, bool addCmt=false,
       warnedNeg=true;
     } 
   }
+  // REprintf("Time15: %f\n", ((double)(clock() - _lastT0))/CLOCKS_PER_SEC);
+  // _lastT0 = clock();
   return lstF;
 }
